"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EnvStatus = void 0;
var childProcess = require("child_process");
var fetch = require("fetch");
var fs = require("fs");
var os = require("os");
var path = require("path");
var EnvStatus = /** @class */ (function () {
    function EnvStatus() {
        this.envDataCache = {};
    }
    EnvStatus.getShared = function () {
        return EnvStatus.instance || (EnvStatus.instance = new EnvStatus());
    };
    EnvStatus.prototype.setConfig = function (config) {
        return (this.envConfig = config);
    };
    EnvStatus.prototype.getConfig = function () {
        if (this.envConfig) {
            return this.envConfig;
        }
        var configPath = path.resolve('.envstatus.js');
        if (!fs.existsSync(configPath)) {
            return null;
        }
        return this.setConfig(require(configPath));
    };
    EnvStatus.prototype.getArgs = function (argv) {
        return argv.slice(2);
    };
    EnvStatus.prototype.fetchOrigin = function () {
        var _this = this;
        if (this.fetchOriginPromise) {
            return this.fetchOriginPromise;
        }
        return (this.fetchOriginPromise = new Promise(function (resolve, reject) {
            childProcess.execFile('git', ['fetch', 'origin'], function (err) {
                if (err) {
                    _this.fetchOriginPromise = null;
                    reject(err);
                }
                else {
                    resolve();
                }
            });
        }));
    };
    EnvStatus.prototype.getLastCommit = function (now) {
        var jsonStr;
        try {
            jsonStr = childProcess
                .execFileSync('git', ['show', '--stat', '--format={"commit": "%h", "author": "%an", "branch": "%d"}|||'])
                .toString()
                .split('|||')[0];
        }
        catch (err) {
            jsonStr = fs
                .readFileSync('last-commit.txt')
                .toString()
                .split('|||')[0];
        }
        var res = JSON.parse(jsonStr);
        res.date = now.getTime();
        res.branch = res.branch.match(/\S*?(?=\))/)[0];
        return res;
    };
    EnvStatus.prototype.getOriginBranch = function (branchName) {
        return branchName.startsWith('origin/') ? branchName : 'origin/' + branchName;
    };
    EnvStatus.prototype.getBranchLastCommitId = function (branchName, cwd) {
        return childProcess
            .execFileSync('git', ['rev-parse', '--short', branchName], { cwd: cwd })
            .toString()
            .trim();
    };
    EnvStatus.prototype.isAncestorCommit = function (c1, c2) {
        if (!c1 || !c2) {
            return false;
        }
        try {
            childProcess.execFileSync('git', ['merge-base', '--is-ancestor', c1, c2]);
            return true;
        }
        catch (err) {
            return false;
        }
    };
    EnvStatus.prototype.getBranchName = function () {
        var res = childProcess
            .execFileSync('git', ['branch'])
            .toString()
            .split(os.EOL)
            .find(function (x) { return x.startsWith('*'); });
        if (res) {
            return res.slice(1).trim();
        }
        else {
            return '';
        }
    };
    EnvStatus.prototype.getBranchType = function (branch) {
        branch = branch.replace(/^origin\//, '');
        if (branch === 'master') {
            return "MASTER" /* MASTER */;
        }
        else if (branch.startsWith('sprint/')) {
            return "SPRINT" /* SPRINT */;
        }
        else if (branch.startsWith('feat/')) {
            return "SPRINT_FEATURE" /* SPRINT_FEATURE */;
        }
        else if (branch.startsWith('fix/')) {
            return "SPRINT_FIX" /* SPRINT_FIX */;
        }
        else if (branch.startsWith('hotfix/')) {
            return "HOTFIX" /* HOTFIX */;
        }
        else {
            return "OTHERS" /* OTHERS */;
        }
    };
    EnvStatus.prototype.getEnvType = function (env) {
        if (env === void 0) { env = ''; }
        if (env === 'production') {
            return "PRODUCTION" /* PRODUCTION */;
        }
        else if (env === 'staging') {
            return "STAGING" /* STAGING */;
        }
        else if (env === 'test') {
            return "TEST" /* TEST */;
        }
        else if (env === 'development') {
            return "DEV" /* DEV */;
        }
        else {
            return "OTHERS" /* OTHERS */;
        }
    };
    EnvStatus.prototype.setEnvDataCache = function (env, data) {
        this.envDataCache[env] = data;
    };
    EnvStatus.prototype.appendCurrentTimestampToUrl = function (url, now) {
        return "" + url + (url.indexOf('?') > 0 ? '&' : '?') + "t=" + now.getTime();
    };
    EnvStatus.prototype.fetchEnvData = function (env) {
        var _this = this;
        return new Promise(function (resolve) {
            if (_this.envDataCache[env]) {
                resolve(_this.envDataCache[env]);
                return;
            }
            var config = _this.getConfig();
            if (!config) {
                resolve({ err: "CONFIG_UNDEFINED" /* CONFIG_UNDEFINED */, env: env });
                return;
            }
            if (typeof config.url !== 'function') {
                resolve({ err: "URL_FUNCTION_UNDEFINED" /* URL_FUNCTION_UNDEFINED */, env: env });
                return;
            }
            var url = config.url(env);
            fetch.fetchUrl(_this.appendCurrentTimestampToUrl(url, new Date()), function (error, meta, body) {
                var data;
                if (error || meta.status !== 200) {
                    data = { err: "LOAD_ERROR" /* LOAD_ERROR */, env: env };
                }
                else {
                    try {
                        data = Object.assign({ env: env }, JSON.parse(body.toString()));
                    }
                    catch (err) {
                        data = { err: "PARSE_RESPONSE_ERROR" /* PARSE_RESPONSE_ERROR */, env: env };
                    }
                }
                _this.setEnvDataCache(env, data);
                resolve(data);
            });
        });
    };
    EnvStatus.prototype.isEnvAvailable = function (env) {
        var _this = this;
        return Promise.all(['production', 'staging', env].map(function (e) { return _this.fetchEnvData(e); })).then(function (envsData) {
            var envData = envsData[2];
            var stgData = envsData[1];
            var prdData = envsData[0];
            return _this.fetchOrigin().then(function () {
                if (envData.env === 'production') {
                    return true;
                }
                else {
                    if (_this.isAncestorCommit(envData.commit, prdData.commit)) {
                        return true;
                    }
                    else if (envData.env === 'staging') {
                        return false;
                    }
                    else {
                        return _this.isAncestorCommit(envData.commit, stgData.commit);
                    }
                }
            });
        });
    };
    return EnvStatus;
}());
exports.EnvStatus = EnvStatus;
